'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.migrateState = migrateState;

var _constants = require('./constants');

function migrateState() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var migrations = arguments[1];
  var currentVersion = arguments[2];
  var _ref = arguments[3];
  var debug = _ref.debug;

  var inboundVersion = state && state.version || _constants.DEFAULT_VERSION;
  if (inboundVersion === currentVersion) {
    if (process.env.NODE_ENV !== 'production' && debug) console.log('redux-persist: verions match, noop migration');
    return state;
  }
  if (inboundVersion > currentVersion) {
    if (process.env.NODE_ENV !== 'production' && debug) console.error('redux-persist: downgrading version is not supported');
    return state;
  }

  var migrationKeys = Object.keys(migrations).map(function (ver) {
    return parseInt(ver);
  }).filter(function (key) {
    return key > inboundVersion;
  }).sort();

  if (process.env.NODE_ENV !== 'production' && debug) console.log('redux-persist: migrationKeys', migrationKeys);
  var migratedState = migrationKeys.reduce(function (state, versionKey) {
    if (process.env.NODE_ENV !== 'production' && debug) console.log('redux-persist: running migration for versionKey', versionKey);
    return migrations[versionKey](state);
  }, state);

  return migratedState;
}